/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */

#ifndef _AFLIBAUDIO_H
#define _AFLIBAUDIO_H

#include "aflib.h"
#include "aflibChain.h"
#include "aflibConfig.h"


// This is the Base class that all objects will use that want to be used
// as audio object with this library. It inherts functionality from the
// afChain class so that these objects can be used in a chain. Anything
// that would be common between all audio objects should be put in this
// class. 
//
// Two constructors are provided. A constructor with no parameters is
// provided that derived classes should call that can be used as the
// start of a chain. A mixing class for instance makes no sense to
// have a default contructor since it can't be used as the start of a
// chain. Most often the start of a chain will be a file or device
// object. The second constructor takes an afAudio object. This should
// be called by a derived class that could have a parent audio object
// feeding it. This will more than likely be almost all objects.
//
// compute_segment is a pure virtual so that all derived classes will
// have to implement it. This is how data is passed to the object to be
// operated on.
//
// process is defined in this base class and should not be overriden except
// in certain circumstances. Such as an audio class with more than 1 input. It
// provides the functionallity to push data throught the chain. The user
// application can call process from any element in the chain. It will
// work its way back through the chain and when it gets to the start will
// start an afData object through the chain until arriving at and processing
// this object. This function might be the ideal place in the future to
// enable threading. 


class aflibData;

class aflibAudio : public aflibChain {

public:

   aflibAudio();
   aflibAudio(const aflibAudio& audio);

   virtual
   ~aflibAudio();

   void
   enable(bool enable);

   bool
   getEnable() const;

   void
   setAudioConfig(const aflibConfig& cfg);

   const aflibConfig&
   getAudioConfig() const;

   virtual aflibData *
   process(
      aflibStatus& ret_status,
      long long position,
      int num_samples,
      bool free_memory = TRUE) const;

   virtual aflibStatus
   compute_segment(
      aflibData& data,
      long long position = -1) = 0;

protected:

private:

bool     _enable;
aflibConfig _cfg;

};


#endif
