/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#include <stdio.h>
#include <iostream.h>

#include "aflibAudioClip.h"


aflibAudioClip::aflibAudioClip()
{
   _input_id = 0;
   _output_start_samples = 0;
   _output_stop_samples = 0;
   _input_start_samples = 0;
   _input_stop_samples = 0;
}

aflibAudioClip::aflibAudioClip(
      int  input,
      long long start_samples_o,
      long long stop_samples_o,
      long long start_samples_i,
      long long stop_samples_i) :
   _input_id(input),
   _output_start_samples(start_samples_o),
   _output_stop_samples(stop_samples_o),
   _input_start_samples(start_samples_i),
   _input_stop_samples(stop_samples_i)
{
}

bool
aflibAudioClip::operator < (const aflibAudioClip& clip) const
{
   return (getStartSamplesOutput() < clip.getStartSamplesOutput());
}

aflibAudioClip::~aflibAudioClip()
{
}

void
aflibAudioClip::setInput(int input)
{
   _input_id = input;
}
   
int
aflibAudioClip::getInput() const
{
   return(_input_id);
}
   
void
aflibAudioClip::setStartSamplesOutput(long long start_samples_o)
{
   _output_start_samples = start_samples_o;
}
   
long long
aflibAudioClip::getStartSamplesOutput() const
{
   return (_output_start_samples);
}
   
void
aflibAudioClip::setStopSamplesOutput(long long stop_samples_o)
{
   _output_stop_samples = stop_samples_o;
}
   
long long
aflibAudioClip::getStopSamplesOutput() const
{
   return (_output_stop_samples);
}
   
void
aflibAudioClip::setStartSamplesInput(long long start_samples_i)
{
   _input_start_samples = start_samples_i;
}
   
long long
aflibAudioClip::getStartSamplesInput() const
{
   return (_input_start_samples);
}

void
aflibAudioClip::setStopSamplesInput(long long stop_samples_i)
{
   _input_stop_samples = stop_samples_i;
}

long long
aflibAudioClip::getStopSamplesInput() const
{
   return (_input_stop_samples);
}

int
aflibAudioClip::compare(const aflibAudioClip& clip)
{
   // This function will compare an audio clip with this one. It will return
   // one of three possible values. 
   // -1 : "this" clip is before "clip" clip sequentially. 
   //  0 : "this" starts within "clip" clip
   //  1 : "this" clip is after "clip" clip

   int ret_value;

   if (getStartSamplesOutput() <= clip.getStartSamplesOutput())
      ret_value = -1;
   else if (getStartSamplesOutput() < clip.getStopSamplesOutput())
      ret_value = 0;
   else
      ret_value = 1;

   return(ret_value);
}


