/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#include "aflibAudioFile.h"
#include "aflibFile.h"

#include <stdio.h>
#include <stdlib.h>

aflibAudioFile::aflibAudioFile(
   aflibFileType type_enum,
   const string& file,
   aflibConfig* cfg,
   aflibStatus* status) : aflibAudio()
{
   _has_parent = FALSE;
   createAfFileObject(type_enum, file, cfg, status);

   if (cfg)
   {
      setAudioConfig(*cfg);
   }
}

aflibAudioFile::aflibAudioFile(
   const aflibAudio& audio,
   aflibFileType type_enum,
   const string& file,
   aflibConfig* cfg,
   aflibStatus* status) : aflibAudio(audio)
{
   _has_parent = TRUE;
   createAfFileObject(type_enum, file, cfg, status);
}


aflibAudioFile::~aflibAudioFile()
{
   delete _file_object;
}

void
aflibAudioFile::createAfFileObject(
   aflibFileType type_enum,
   const string& file,
   aflibConfig* cfg,
   aflibStatus* status)
{
   // File object can only be at start of end of chain. If a parent is present then
   // must be at end of chain (ie write to file).
   if (_has_parent == TRUE)
   {
      // TBD if cfg is NULL need to get from previous node
      _file_object = aflibFile::create(type_enum, file, *cfg, status);
   }
   else
   {
      _file_object = aflibFile::open(type_enum, file, cfg, status);
   }
}

aflibStatus
aflibAudioFile::compute_segment(
   aflibData& data,
   long long position)
{
   aflibStatus ret_status;

   if (_has_parent == TRUE)
   {
      ret_status = _file_object->afwrite(data, position);
   }
   else
   {
      ret_status = _file_object->afread(data, position);
   }
   return (ret_status);
}


const aflibConfig&
aflibAudioFile::getConfig() const
{
   return (_file_object->getConfig());
}


