/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#ifndef _AFLIBAUDIOFILE_H
#define _AFLIBAUDIOFILE_H

#include "aflib.h"
#include "aflibAudio.h"
#include "bstring.h"


// This class is the object wrapper that can make an aflibFile object useable
// as an audio object and to be used in a chain. For using file and device
// objects in a chain this is the API that one should use. The constructors
// are the same as the base class. We may need to change the API's in the
// future. For instance remove the string object and use a char * instead
// so that Java, C, and other languages other than C++ can use this
// library. I don't have much experience in this area yet. 
//
// cumpute_segment is implemented so that the base aflibAudio class can
// process data throught this class. 

// May want to make sure in the future that this class is not in the middle of a
// chain. Another words these all either output or input data internally we can't
// pass data through like for instance a filter.

class aflibConfig;
class aflibFile;

class aflibAudioFile : public aflibAudio {

public:

   // Constructor for a node with no parent (ie start of a chain)
   aflibAudioFile(
      aflibFileType type_enum,
      const string& file,
      aflibConfig* cfg = NULL,
      aflibStatus* status = NULL);

   // Constructor for a node with a parent
   aflibAudioFile(
      const aflibAudio& audio,
      aflibFileType type_enum,
      const string& file,
      aflibConfig* cfg = NULL,
      aflibStatus* status = NULL);

   ~aflibAudioFile();

   aflibStatus
   compute_segment(
      aflibData& data,
      long long position = -1) ;

   const aflibConfig&
   getConfig() const;

protected:

private:

   void
   createAfFileObject(
      aflibFileType type_enum,
      const string& file,
      aflibConfig* cfg = NULL,
      aflibStatus* status = NULL);


aflibFile  *  _file_object;
bool       _has_parent;

};


#endif
