/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#ifndef _AFLIBAUDIORECORDER_H_
#define _AFLIBAUDIORECORDER_H_

#include "aflibAudio.h"
#include "aflibDateTime.h"
#include "aflibRecorderItem.h"
#include "bstring.h"

#include <list.h>

class aflibConfig;


class aflibAudioRecorder : public aflibAudio {

public:

   // Available contructors and destructors
   aflibAudioRecorder(const aflibAudio& audio);
   ~aflibAudioRecorder();

   void
   addRecordItem(
      const aflibDateTime& start_date,
      const aflibDateTime& stop_date,
      const string& file,
      aflibFileType file_type,
      long long max_limit,
      long long each_limit,
      aflibConfig& config);

   void
   removeRecordItem(
      int item);

   int
   getNumberOfRecordItems() const;

   void
   getItem(
      int item,
      aflibDateTime& start_date,
      aflibDateTime& stop_date,
      string& file,
      aflibFileType& file_type,
      long long& max_limit,
      long long& each_limit,
      aflibConfig& config) const;

   aflibStatus
   compute_segment(
      aflibData& data,
      long long position = -1) ;

private:

   aflibAudioRecorder();

   aflibAudioRecorder(const aflibAudioRecorder& op);

   const aflibAudioRecorder&
   operator=(const aflibAudioRecorder& op);

   aflibDateTime
   getCurrentTime();

   bool
   audioFileSizeCheck(aflibRecorderItem& item);

list<aflibRecorderItem>   _item_list;

};


#endif
