/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#include "aflibChain.h"
#include "aflibChainNode.h"

#include <stdio.h>
#include <stdlib.h>


list<aflibChainNode *>  aflibChain::_start_list;
list<aflibChainNode *>  aflibChain::_total_list;


aflibChain::aflibChain()
{
   _this_node = NULL;

   insertIntoChain(*((aflibAudio*)this));
}

aflibChain::aflibChain(
   const aflibAudio& parent)
{
   _this_node = NULL;

   insertIntoChain(parent, *((aflibAudio*)this));
}

aflibChain::~aflibChain()
{
   removeFromChain(*((aflibAudio*)this));

   delete _this_node;
}

const list<const aflibAudio *>&
aflibChain::getParents() const
{
   if (_this_node == NULL)
   {
      cerr << "SOFTWARE ERROR: aflibChain::getParents()" << endl;
      exit(1);
   }
   return (_this_node->getParents());
}


void
aflibChain::insertIntoChain(
   const aflibAudio& parent,
   const aflibAudio& child)
{
   _this_node = new aflibChainNode(child);

   _this_node->addParent(parent);
   cout << "Inserted new node into chain " << _this_node << endl;

   _total_list.push_back(_this_node);
}

void
aflibChain::insertIntoChain(
   const aflibAudio& child)
{
  // Use this as a start node
   _this_node = new aflibChainNode(child);
   cout << "Inserted new node into chain " << _this_node << endl;

   _start_list.push_back(_this_node);
   _total_list.push_back(_this_node);
}

void
aflibChain::removeFromChain(
   const aflibAudio& child)
{
   list<aflibChainNode *>::iterator it;

   // Find the node in the total node list and remove
   for (it = _total_list.begin(); it != _total_list.end(); it++)
   {
      if (&child == &((*it)->getAudioItem()))
      {
         cout << "Removed node from chain " << *it << endl;
         _total_list.erase(it);
         break;
      }
   }

   // Find the node in the start node list and remove
   for (it = _start_list.begin(); it != _start_list.end(); it++)
   {
      if (&child == &((*it)->getAudioItem()))
      {
         cout << "Removed node from from start list" << *it << endl;
         _start_list.erase(it);
         break;
      }
   }
}


int
aflibChain::addParent(const aflibAudio& parent)
{
   // If an object wishes to add another item as a parent we will let them.
   // Some objects need more than one input. This provides for it.

   int ret_id;

   if (_this_node == NULL)
   {
      cerr << "SOFTWARE ERROR: aflibChain::addParents()" << endl;
      exit(1);
   }

   ret_id = _this_node->addParent(parent);

   return(ret_id);
}

void
aflibChain::removeParent(int parent_id)
{
   // If an object wishes to remove a parent then we will let them. Some
   // objects have more than one input and may want to remove one or more.
   // NOTE: we are not talking about the node being deleted just the chain
   // between this child and parent

   if (_this_node == NULL)
   {
      cerr << "SOFTWARE ERROR: aflibChain::removeParents()" << endl;
      exit(1);
   }

   _this_node->removeParent(parent_id);
}

void
aflibChain::parentWasDestroyed(int parent_id)
{
}




