/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


#ifndef _AFLIBCHAIN_H
#define _AFLIBCHAIN_H

#include "list.h"
#include "aflib.h"

// This is the base class that provides the mechanism to chain audio objects
// together. It maintains several static lists that contain all of the 
// chain information for this application. Alot more code will need to be
// added to this class in the future. For instance if someone were to
// destroy an object in the middle of a chain we need to provide functionality
// that will keep the chain active and connected by connecting the previous
// and next nodes. 
//
// This class provides a default contructor for the start of a chain and
// a constructor that takes an aflibAudio object as the parent of this object.
// 
// getParents is provided so that users can get the parents of this node. For
// now we assume only one parent per node. But a list is returned so that
// in the future many nodes could be returned. 
//
//

class aflibAudio;
class aflibChainNode;


class aflibChain {

public:

   aflibChain();
   aflibChain(const aflibAudio& parent);

   virtual
   ~aflibChain();

   const list<const aflibAudio *>&
   getParents() const;

   int
   addParent(const aflibAudio& parent);

   void
   removeParent(int parent_id);

   // This is a callback function that derived classes can over ride to be
   // notified when a parent was destroyed. An Audio Editor object
   // for instance may have many inputs and if one is destroyed then it needs to
   // be notified so that it can rebuild its lists.
   virtual void
   parentWasDestroyed(int parent_id);

protected:

private:

   void
   insertIntoChain(
      const aflibAudio& parent,
      const aflibAudio& child);

   void
   insertIntoChain(
      const aflibAudio& child);

   void
   removeFromChain(
      const aflibAudio& child);

static list<aflibChainNode *>  _start_list;  
static list<aflibChainNode *>  _total_list;  
aflibChainNode *   _this_node;

};


#endif


