// This code was obtained from code with the following header
/*============================================================================

    Don Cross <dcross@intersrv.com>

    http://www.intersrv.com/~dcross/fft.html

    Contains definitions for doing Fourier transforms
    and inverse Fourier transforms.

    Revision history:

1998 September 19 [Don Cross]
    Updated coding standards.
    Improved efficiency of trig calculations.

============================================================================*/

// This class provides a FFT for other classes in this library to use. There is
// only one API for this class and it is fft_double. It will perform both a
// forward and reverse FFT. It operates on doubles.


#ifndef _AFLIBFFT_H
#define _AFLIBFFT_H

#include "aflib.h"



class aflibFFT {

public:

   aflibFFT();

   ~aflibFFT();

   void
   fft_double (
      unsigned  NumSamples,
      int       InverseTransform,
      const double   *RealIn,
      const double   *ImagIn,
      double   *RealOut,
      double   *ImagOut );

private:

   void
   CheckPointer (
      const void *p,
      const char *name );

   int
   IsPowerOfTwo ( unsigned x );

   unsigned
   NumberOfBitsNeeded ( unsigned PowerOfTwo );

   unsigned
   ReverseBits (
      unsigned index,
      unsigned NumBits );

   double
   Index_to_frequency (
      unsigned NumSamples,
      unsigned Index );

};


#endif
