/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


// Base class for audio file reading and writing

#include "aflibFile.h"

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#ifdef NO_MODULES
#include <aflibDevFile.h>
#include <aflibMp3File.h>
#include <aflibWavFile.h>
#endif

aflibFile::aflibFile(int num)
{

   _lib1 = NULL;

#ifndef NO_MODULES 
   const char * error_msg;

   if (num == 1)
   {
      strcpy(_lib_name, "aflibWavFile");
      _lib1 = dlopen("libaflibwav.so", RTLD_NOW );
   }
   else if (num == 2)
   {
      strcpy(_lib_name, "aflibMp3File");
      _lib1 = dlopen("libaflibmp3.so", RTLD_NOW );
   }
   else if (num == 3)
   {
      strcpy(_lib_name, "aflibDevFile");
      _lib1 = dlopen("libaflibdev.so", RTLD_NOW );
   }
   if (_lib1 == NULL)
   {
      cerr << " lib1 is NULL" << endl;
   }
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }


   void (*func_ptr)() ;
   char buf[256];

   sprintf(buf, "__9%s",_lib_name);
   func_ptr = (void(*)()) dlsym(_lib1, buf);
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }
   if (func_ptr == NULL)
   {
      cerr << "NULL PTR" << endl;
   }
   else
   {
    func_ptr();
   }
#endif
}

aflibFile::aflibFile()
{
   _lib1 = NULL;
}

aflibFile::~aflibFile()
{
   if (_lib1)
   {
      dlclose(_lib1);
   }
}


aflibFile *
aflibFile::open(
   aflibFileType type_enum,
   const string& file,
   aflibConfig* cfg,
   aflibStatus* status)
{
   aflibFile * file_return = NULL;
   aflibStatus ret_status = AFLIB_ERROR_OPEN;


#ifndef NO_MODULES
   if (type_enum == AFLIB_DEV_TYPE)
   {
      file_return = new aflibFile(3);
   }
   else if (type_enum == AFLIB_MPEG_TYPE)
   {
      file_return = new aflibFile(2);
   }
   else if (type_enum == AFLIB_WAV_TYPE)
   {
      file_return = new aflibFile(1);
   }
   else
   {
      ret_status = AFLIB_ERROR_UNSUPPORTED;
   }
#else
   if (type_enum == AFLIB_DEV_TYPE)
   {
      file_return = new aflibDevFile();
   }
   else if (type_enum == AFLIB_MPEG_TYPE)
   {
      file_return = new aflibMp3File();
   }
   else if (type_enum == AFLIB_WAV_TYPE)
   {
      file_return = new aflibWavFile();
   }
   else
   {
      ret_status = AFLIB_ERROR_UNSUPPORTED;
   }
#endif

   if (file_return)
   {
      ret_status = file_return->afopen(file, cfg);

      if (cfg)
      {
         *cfg = file_return->getConfig();
      }
   }

   if (status != NULL)
   {
      *status = ret_status;
   }
   return (file_return);
}

aflibFile *
aflibFile::create(
   aflibFileType type_enum,
   const string& file,
   const aflibConfig& cfg,
   aflibStatus* status)
{
   aflibFile * file_return = NULL;
   aflibStatus ret_status = AFLIB_ERROR_OPEN;


#ifndef NO_MODULES
   if (type_enum == AFLIB_DEV_TYPE)
   {
      file_return = new aflibFile(3);
   }
   else if (type_enum == AFLIB_MPEG_TYPE)
   {
      file_return = new aflibFile(2);
   }
   else if (type_enum == AFLIB_WAV_TYPE)
   {
      file_return = new aflibFile(1);
   }
   else
   {
      ret_status = AFLIB_ERROR_UNSUPPORTED;
   }
#else
   if (type_enum == AFLIB_DEV_TYPE)
   {
      file_return = new aflibDevFile();
   }
   else if (type_enum == AFLIB_MPEG_TYPE)
   {
      file_return = new aflibMp3File();
   }
   else if (type_enum == AFLIB_WAV_TYPE)
   {
      file_return = new aflibWavFile();
   }
   else
   {
      ret_status = AFLIB_ERROR_UNSUPPORTED;
   }
#endif

   if (file_return)
   {
      ret_status = file_return->afcreate(file, cfg);
   }

   if (status != NULL)
   {
      *status = ret_status;
   }
   return (file_return);
}

bool
aflibFile::isSizeSupported(
   aflib_data_size data_size)
{
   // This defines what format this class will support data in.

   // Default to NOTHING supported
   return (FALSE);
}

bool
aflibFile::isEndianSupported(
   aflib_data_endian data_end)
{
   // This defines what format this class will support data in.

   // Default to NOTHING supported
   return (FALSE);
}

const aflibConfig&
aflibFile::getConfig() const
{
   return (_cfg);
}

aflibStatus
aflibFile::afopen(
   const string& file,
   aflibConfig* cfg)
{
   aflibStatus status = AFLIB_SUCCESS;
   
#ifndef NO_MODULES
   const char * error_msg;
   aflibStatus (*func_ptr)(void *, const string&, afConfig*) ;
   char buf[256];
   if (cfg)
   {
      _cfg = *cfg;
   }

   sprintf(buf, "afopen__9%sRCt12basic_string1ZcP8aflibConfig",_lib_name);
   func_ptr = (aflibStatus(*)(void *, const string&, aflibConfig*)) dlsym(_lib1, buf);
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }
   if (func_ptr == NULL)
   {
      cerr << "NULL PTR" << endl;
   }
   else
   {
     status  = func_ptr(this, file, cfg);
   }
#endif

   return (status);
}

aflibStatus
aflibFile::afcreate(
   const string& file,
   const aflibConfig& cfg)
{
   aflibStatus status = AFLIB_SUCCESS;
   

#ifndef NO_MODULES
   const char * error_msg;
   aflibStatus (*func_ptr)(void *, const string&, const aflibConfig&) ;
   char buf[256];
   sprintf(buf, "afcreate__9%sRCt12basic_string1ZcRC8aflibConfig",_lib_name);
   func_ptr = (aflibStatus (*)(void *, const string&, const aflibConfig&)) dlsym(_lib1, buf);
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }
   if (func_ptr == NULL)
   {
      cerr << "NULL PTR" << endl;
   }
   else
   {
     status  = func_ptr(this, file, cfg);
   }
#endif

   return (status);
}

aflibStatus
aflibFile::afread(
   aflibData& data,
   long long position)
{
   aflibStatus status = AFLIB_SUCCESS;
   

#ifndef NO_MODULES
   const char * error_msg;
   aflibStatus (*func_ptr)(void *, aflibData&, long) ;
   char buf[256];
   sprintf(buf, "afread__9%sR6aflibDatal",_lib_name);
   func_ptr = (aflibStatus (*)(void *, aflibData&, long)) dlsym(_lib1, buf);
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }
   if (func_ptr == NULL)
   {
      cerr << "NULL PTR" << endl;
   }
   else
   {
     status  = func_ptr(this, data, position);
   }
#endif

   return (status);
}

aflibStatus
aflibFile::afwrite(
   aflibData& data,
   long long position)
{
   aflibStatus status = AFLIB_SUCCESS;
   

#ifndef NO_MODULES
   const char * error_msg;
   aflibStatus (*func_ptr)(void *, aflibData&, long) ;
   char buf[256];
   sprintf(buf, "afwrite__9%sR6aflibDatal",_lib_name);
   func_ptr = (aflibStatus (*)(void *, aflibData&, long)) dlsym(_lib1, buf);
   error_msg = dlerror();
   if (error_msg)
   {
      cerr << "Error is : " << error_msg << endl;
   }
   if (func_ptr == NULL)
   {
      cerr << "NULL PTR" << endl;
   }
   else
   {
     status  = func_ptr(this, data, position);
   }
#endif

   return (status);
}



