/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */

#include <stdio.h>

#include "aflib.h"
#include "aflibRecorderItem.h"


aflibRecorderItem::aflibRecorderItem() :
   _max_limit(-1),
   _each_limit(-1),
   _current_size(0),
   _audio_file(NULL),
   _file_segment(0)
{
}

aflibRecorderItem::aflibRecorderItem(
   const aflibDateTime start_date,
   const aflibDateTime stop_date,
   const string& file,
   aflibFileType file_type,
   long long max_limit,
   long long each_limit,
   aflibConfig& config) :
   _start_date(start_date),
   _stop_date(stop_date),
   _file(file),
   _base_file(file),
   _file_type(file_type),
   _max_limit(max_limit),
   _each_limit(each_limit),
   _current_size(0),
   _audio_file(NULL),
   _file_segment(0),
   _config(config)
{
}

aflibRecorderItem::~aflibRecorderItem()
{
}

void
aflibRecorderItem::setStartDate(const aflibDateTime& date1)
{
   _start_date = date1;
}

const aflibDateTime&
aflibRecorderItem::getStartDate() const
{
   return (_start_date);
}

void
aflibRecorderItem::setStopDate(const aflibDateTime& date1)
{
   _stop_date = date1;
}

const aflibDateTime&
aflibRecorderItem::getStopDate() const
{
   return (_stop_date);
}

void
aflibRecorderItem::setAudioFile(const string& file)
{
   _file = file;
}

const string&
aflibRecorderItem::getAudioFile() const
{
   return (_file);
}

void
aflibRecorderItem::setAudioFileType(aflibFileType file_type)
{
   _file_type = file_type;
}

aflibFileType
aflibRecorderItem::getAudioFileType() const
{
   return (_file_type);
}

void
aflibRecorderItem::setMaxFileLimit(long long size)
{
   _max_limit = size;
}

long long
aflibRecorderItem::getMaxFileLimit() const
{
   return(_max_limit);
}

void
aflibRecorderItem::setEachFileLimit(long long size)
{
   _each_limit = size;
}

long long
aflibRecorderItem::getEachFileLimit() const
{
   return (_each_limit);
}

void
aflibRecorderItem::setCurrentSize(long long size)
{
   _current_size = size;
}

long long
aflibRecorderItem::getCurrentSize() const
{
   return (_current_size);
}

void
aflibRecorderItem::setFileObject(aflibAudioFile * audio)
{
   _audio_file = audio;
}

aflibAudioFile *
aflibRecorderItem::getFileObject() const
{
   return (_audio_file);
}

void
aflibRecorderItem::setConfig(const aflibConfig& cfg)
{
   _config = cfg;
}

const aflibConfig&
aflibRecorderItem::getConfig() const
{
   return (_config);
}

void
aflibRecorderItem::processNextFile()
{
   size_t insert_position;
   char buf[100];

   _file_segment++;

   sprintf(buf, "%d", _file_segment);
   insert_position = _base_file.rfind('.');
   _base_file.insert(insert_position, buf, strlen(buf));
   setAudioFile(_base_file);
}



