// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream.h>

#include <X11/X.h>

extern "C"
{
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/DrawingA.h>
}

#include "aflibConfig.h"
#include "aflibAudioFile.h"
#include "aflibData.h"
#include "aflibSampleData.h"
#include "bstring.h"


int start_flag = 0;
int num_channels = 1;
int bit_width = 8;
int data_rate = 22050;
aflibFileType input_format = AFLIB_DEV_TYPE;
aflibFileType output_format = AFLIB_MPEG_TYPE;
aflib_data_quality quality = AFLIB_QUALITY_MED;
static Widget InputText, OutputText;


static XrmOptionDescRec opTable[] =
{
   {"-time", ".time", XrmoptionSepArg, NULL},
};

static String FallBack[] =
{
   "*borderWidth: 1",
   "*geometry: +100+100",
   "*background: grey",
   NULL
};



void
start_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   cout << "Start Got Called " << endl;

   start_flag = 1;
}


void
stop_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   cout << "Stop Got Called" << endl;

   start_flag = 0;
}

void
mono_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Mono Got Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      num_channels = 1;
   }
   else
   {
      num_channels = 2;
   }
}

void
stereo_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   cout << "Stereo Got Called" << endl;
}

void
bit8_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "8 Bit Got Called " << call_data->set << endl;
   if (call_data->set == 1)
   {
      bit_width = 8;
   }
   else
   {
      bit_width = 16;
   }
}

void
bit16_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   cout << "16 Bit Got Called" << endl;
}

void
rate_slow_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "22050 rate Got Called " << call_data->set << endl;
   if (call_data->set == 1)
   {
      data_rate = 22050;
   }
   else
   {
      data_rate = 44100;
   }
}

void
rate_fast_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   cout << "44100 Rate Got Called" << endl;
}

void
input_device_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Input Device Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      input_format = AFLIB_DEV_TYPE;
   }
}

void
input_mpeg_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Input MPEG Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      input_format = AFLIB_MPEG_TYPE;
   }
}

void
input_wav_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Input WAVE Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      input_format = AFLIB_WAV_TYPE;
   }
}

void
output_device_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Output DEVICE Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      output_format = AFLIB_DEV_TYPE;
   }
}

void
output_mpeg_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Output MPEG Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      output_format = AFLIB_MPEG_TYPE;
   }
}

void
output_wav_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Output WAVE Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      output_format = AFLIB_WAV_TYPE;
   }
}

void
quality_low_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Quality low Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      quality = AFLIB_QUALITY_LOW;
   }
}

void
quality_med_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Quality med Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      quality = AFLIB_QUALITY_MED;
   }
}

void
quality_high_call(
   Widget w,
   XtPointer client_data,
   XmToggleButtonCallbackStruct* call_data)
{
   cout << "Quality high Called " << call_data->set << endl;
   if (call_data->set == True)
   {
      quality = AFLIB_QUALITY_HIGH;
   }
}


void
update_data(bool stop)
{
   static bool initialized = 0;
   static aflibConfig config;
   static aflibAudioFile * output = NULL;
   static aflibAudioFile * input = NULL;
   static long long position = 0;
   aflibStatus status = AFLIB_SUCCESS;
   char *in_text = NULL;
   char *out_text = NULL;
   string in_str;
   string out_str;

   if (stop == 0)
   {
      delete output;
      output = NULL;
      delete input;
      input = NULL;
      initialized = 0;
      usleep(1);
      position = 0;
      return;
   }
   if (initialized == 0)
   {
      delete output;
      delete input;

      config.setChannels(num_channels);
      if (bit_width == 8)
      {
         config.setSampleSize(AFLIB_DATA_8S);
      }
      else if (bit_width == 16)
      {
         config.setSampleSize(AFLIB_DATA_16U);
      }
      else
      {
         cerr << "Invalid Bits Per Sample" << endl;
      }
      config.setSamplesPerSecond(data_rate);
      config.setDataQuality(quality);
      in_text = XmTextGetString(InputText);
      in_str = in_text;
      cout << "Input text is " << in_text << endl;
      out_text = XmTextGetString(OutputText);
      out_str = out_text;
      cout << "Output text is " << out_text << endl;
      input = new aflibAudioFile(input_format, in_str, &config, &status);
      if (status != AFLIB_SUCCESS)
      {
         cout << "Error opening input device !" << endl;
      }
      cout << "Number of samples is " << config.getTotalSamples() << endl;
      cout << "Number samples / sec " << config.getSamplesPerSecond() << endl;
      cout << "Number of seconds ";
      cout <<  config.getTotalSamples() / config.getSamplesPerSecond() << endl;

      output = new aflibAudioFile(*input, output_format, out_str, &config, &status);
      if (status != AFLIB_SUCCESS)
      {
         cout << "Error opening file !" << endl;
      }

      if (in_text)
         XtFree(in_text);
      if (out_text)
         XtFree(out_text);

      initialized = 1;
   }

   output->process(status, position, 1024);
   position += 1024;
   if (status != AFLIB_SUCCESS)
   {
      start_flag = 0;
   }
}

Boolean
work_app(XtPointer client_data)
{
   update_data(start_flag);

   return False;
}

int
main(int argc, char * argv[])
{
   Widget toplevel, Form, StartAudio, StopAudio;
   Widget Channels, Mono_w, Stereo_w;
   Widget BitWidth, Bit_Width_w[2];
   Widget DataRate, Data_Rate_w[2];
   Widget Quality, Quality_w[3];
   Widget InputFormat, Input_Format_w[3];
   Widget OutputFormat, Output_Format_w[3];
   Widget Label[10];
   XtAppContext  app;
   Arg  args[20];
   int  n;


   toplevel = XtVaAppInitialize(&app, "Form1",
               opTable, XtNumber(opTable),
               &argc, argv,
               FallBack, NULL);

   Form = XmCreateForm(toplevel, "Form", NULL, 0);
   XtVaSetValues(Form,
                 XmNresizePolicy, XmRESIZE_ANY,
                 XmNallowShellResize, True,
                 NULL);



   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
   XtSetArg(args[n], XmNrightPosition, 25); n++;
   Channels = XmCreateRowColumn(Form, "Channels", args, n);
   XtManageChild(Channels);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[0] = XmCreateLabel(Channels, "MODE", args, n);
   XtManageChild(Label[0]);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Mono_w = XmCreateToggleButton(Channels, "Mono", args, n);
   XtManageChild(Mono_w);
   XtAddCallback(Mono_w, XmNvalueChangedCallback, mono_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Stereo_w = XmCreateToggleButton(Channels, "Stereo", args, n);
   XtManageChild(Stereo_w);
   XtAddCallback(Stereo_w, XmNvalueChangedCallback, stereo_call, 0);

   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, Channels); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
   XtSetArg(args[n], XmNrightPosition, 50); n++;
   BitWidth = XmCreateRowColumn(Form, "Bit Width", args, n);
   XtManageChild(BitWidth);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[1] = XmCreateLabel(BitWidth, "BITS", args, n);
   XtManageChild(Label[1]);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Bit_Width_w[0] = XmCreateToggleButton(BitWidth, "8 Bit", args, n);
   XtManageChild(Bit_Width_w[0]);
   XtAddCallback(Bit_Width_w[0], XmNvalueChangedCallback, bit8_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Bit_Width_w[1] = XmCreateToggleButton(BitWidth, "16 Bit", args, n);
   XtManageChild(Bit_Width_w[1]);
   XtAddCallback(Bit_Width_w[1], XmNvalueChangedCallback, bit16_call, 0);


   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, BitWidth); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
   XtSetArg(args[n], XmNrightPosition, 75); n++;
   DataRate = XmCreateRowColumn(Form, "Date Rate", args, n);
   XtManageChild(DataRate);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[2] = XmCreateLabel(DataRate, "RATE", args, n);
   XtManageChild(Label[2]);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Data_Rate_w[0] = XmCreateToggleButton(DataRate, "22050", args, n);
   XtManageChild(Data_Rate_w[0]);
   XtAddCallback(Data_Rate_w[0], XmNvalueChangedCallback, rate_slow_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Data_Rate_w[1] = XmCreateToggleButton(DataRate, "44100", args, n);
   XtManageChild(Data_Rate_w[1]);
   XtAddCallback(Data_Rate_w[1], XmNvalueChangedCallback, rate_fast_call, 0);

   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, DataRate); n++;
   Quality = XmCreateRowColumn(Form, "Quality", args, n);
   XtManageChild(Quality);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[3] = XmCreateLabel(Quality, "QUALITY", args, n);
   XtManageChild(Label[3]);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Quality_w[0] = XmCreateToggleButton(Quality, "Low", args, n);
   XtManageChild(Quality_w[0]);
   XtAddCallback(Quality_w[0], XmNvalueChangedCallback, quality_low_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Quality_w[1] = XmCreateToggleButton(Quality, "Medium", args, n);
   XtManageChild(Quality_w[1]);
   XtAddCallback(Quality_w[1], XmNvalueChangedCallback, quality_med_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Quality_w[2] = XmCreateToggleButton(Quality, "High", args, n);
   XtManageChild(Quality_w[2]);
   XtAddCallback(Quality_w[2], XmNvalueChangedCallback, quality_high_call, 0);

   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, BitWidth); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
   XtSetArg(args[n], XmNrightPosition, 50); n++;
   InputFormat = XmCreateRowColumn(Form, "Input Format", args, n);
   XtManageChild(InputFormat);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[3] = XmCreateLabel(InputFormat, "INPUT FORMAT", args, n);
   XtManageChild(Label[3]);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Input_Format_w[0] = XmCreateToggleButton(InputFormat, "Device", args, n);
   XtManageChild(Input_Format_w[0]);
   XtAddCallback(Input_Format_w[0], XmNvalueChangedCallback, input_device_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Input_Format_w[1] = XmCreateToggleButton(InputFormat, "MPEG", args, n);
   XtManageChild(Input_Format_w[1]);
   XtAddCallback(Input_Format_w[1], XmNvalueChangedCallback, input_mpeg_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Input_Format_w[2] = XmCreateToggleButton(InputFormat, "WAV", args, n);
   XtManageChild(Input_Format_w[2]);
   XtAddCallback(Input_Format_w[2], XmNvalueChangedCallback, input_wav_call, 0);


   n = 0;
   XtSetArg(args[n], XmNradioBehavior, True); n++;
   XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, InputFormat); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, DataRate); n++;
   OutputFormat = XmCreateRowColumn(Form, "Output Format", args, n);
   XtManageChild(OutputFormat);

   n = 0;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[4] = XmCreateLabel(OutputFormat, "OUTPUT FORMAT", args, n);
   XtManageChild(Label[4]);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Output_Format_w[0] = XmCreateToggleButton(OutputFormat, "Device", args, n);
   XtManageChild(Output_Format_w[0]);
   XtAddCallback(Output_Format_w[0], XmNvalueChangedCallback, output_device_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, True); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Output_Format_w[1] = XmCreateToggleButton(OutputFormat, "MPEG", args, n);
   XtManageChild(Output_Format_w[1]);
   XtAddCallback(Output_Format_w[1], XmNvalueChangedCallback, output_mpeg_call, 0);
   n = 0;
   XtSetArg(args[n], XmNset, False); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Output_Format_w[2] = XmCreateToggleButton(OutputFormat, "WAV", args, n);
   XtManageChild(Output_Format_w[2]);
   XtAddCallback(Output_Format_w[2], XmNvalueChangedCallback, output_wav_call, 0);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, InputFormat); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[5] = XmCreateLabel(Form, "Input File", args, n);
   XtManageChild(Label[5]);
   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, Label[5]); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, InputFormat); n++;
   XtSetArg(args[n], XmNeditMode, XmSINGLE_LINE_EDIT); n++;
   XtSetArg(args[n], XmNcolumns, 50); n++;
   InputText = XmCreateText(Form, "Input Text", args, n);
   XtManageChild(InputText);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, InputText); n++;
   XtSetArg(args[n], XmNborderWidth, 0); n++;
   Label[6] = XmCreateLabel(Form, "Output File", args, n);
   XtManageChild(Label[6]);
   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, Label[6]); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, InputText); n++;
   XtSetArg(args[n], XmNeditMode, XmSINGLE_LINE_EDIT); n++;
   XtSetArg(args[n], XmNcolumns, 50); n++;
   OutputText = XmCreateText(Form, "Output Text", args, n);
   XtManageChild(OutputText);


   StartAudio = XmCreatePushButton(Form, "Start Audio", NULL, 0);
   XtVaSetValues(StartAudio,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, OutputText,
                 XmNleftAttachment, XmATTACH_FORM,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 50,
                 NULL);
   XtAddCallback(StartAudio, XmNactivateCallback, start_call, NULL);
   XtManageChild(StartAudio);

   StopAudio = XmCreatePushButton(Form, "Stop Audio", NULL, 0);
   XtVaSetValues(StopAudio,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, OutputText,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, StartAudio,
                 XmNrightAttachment, XmATTACH_FORM,
                 NULL);
   XtAddCallback(StopAudio, XmNactivateCallback, stop_call, NULL);
   XtManageChild(StopAudio);

   XtManageChild(Form);

   XtRealizeWidget(toplevel);

   XtAppAddWorkProc(app, work_app, NULL);

   XtAppMainLoop(app);

}


