// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream.h>

#include <X11/X.h>

extern "C"
{
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
}

#include "plot.h"
#include "gui.h"


static Widget DrawArea;

static XrmOptionDescRec opTable[] =
{
   {"-time", ".time", XrmoptionSepArg, NULL},
};

static String FallBack[] =
{
   "*borderWidth: 0",
   "*geometry: +100+100",
   "*background: grey",
   NULL
};


void
enlarge_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_enlarge();
}

void
reduce_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_reduce();
}

void
shift_left_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_shift_left();
}

void
shift_right_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_shift_right();
}

void
play_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_play(NULL);
}

void
cut_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_cut();
}

void
reset_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_reset();
}

void
erase_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_erase();
}

void
pause_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_pause();
}

void
stop_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   plot_stop();
}

void
gui_save_clip_file(
   Widget w,
   caddr_t client_data,
   XmFileSelectionBoxCallbackStruct  *cbs)
{
   char * file = NULL;

   if (cbs)
   {
      if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file))
         return;

      plot_play(file);
      XtFree(file);
   }
   XtUnmanageChild(w);
}

void
gui_load_file(
   Widget w,
   caddr_t client_data,
   XmFileSelectionBoxCallbackStruct  *cbs)
{
   char * file = NULL;

   if (cbs)
   {
      if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file))
         return;

      plot_call(file);
      XtFree(file);
   }
   XtUnmanageChild(w);
}

void
gui_cancel_file(
   Widget w,
   caddr_t client_data,
   XmFileSelectionBoxCallbackStruct  *cbs)
{
   XtUnmanageChild(w);
}


void
open_callback(
   Widget  w,
   char*   name,
   caddr_t call_data)
{
   Arg  args[10];
   int  n;
   static Widget dialog = 0;

   if (!dialog)
   {
      n = 0;
      XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
      dialog = XmCreateFileSelectionDialog(w, "FileBox", args, n);
      XtAddCallback(dialog, XmNokCallback, gui_load_file, NULL);
      XtAddCallback(dialog, XmNcancelCallback, gui_cancel_file, NULL);
   }
   XtManageChild(dialog);
   XtPopup(XtParent(dialog), XtGrabNone);
}

void
save_clip_callback(
   Widget  w,
   char*   name,
   caddr_t call_data)
{
   Arg  args[10];
   int  n;
   static Widget dialog = 0;

   if (!dialog)
   {
      n = 0;
      XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
      dialog = XmCreateFileSelectionDialog(w, "SaveClipBox", args, n);
      XtAddCallback(dialog, XmNokCallback, gui_save_clip_file, NULL);
      XtAddCallback(dialog, XmNcancelCallback, gui_cancel_file, NULL);
   }
   XtManageChild(dialog);
   XtPopup(XtParent(dialog), XtGrabNone);
}

void
quit_callback(
   Widget  w,
   char*   name,
   caddr_t call_data)
{
   exit(0);
}

void
gui_build(int argc, char * argv[])
{
   Widget toplevel, Form, Enlarge, Reduce, ShiftRight, ShiftLeft;
   Widget MenuBar, FileMenu, FileBar, OpenButton, SaveClipButton, QuitButton;
   Widget PlayWidget, CutWidget, ResetWidget, EraseWidget, PauseWidget, StopWidget;
   XtAppContext  app;
   Arg  args[20];
   int  n;


   toplevel = XtVaAppInitialize(&app, "Form1",
               opTable, XtNumber(opTable),
               &argc, argv,
               FallBack, NULL);

   Form = XmCreateForm(toplevel, "Form", NULL, 0);
   XtVaSetValues(Form,
                 XmNresizePolicy, XmRESIZE_ANY,
                 XmNallowShellResize, True,
                 NULL);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   MenuBar = XmCreateMenuBar(Form, "Menu Bar", args, n);
   XtManageChild(MenuBar);

   n = 0;
   XtSetArg(args[n], XmNmnemonic, 'F'); n++;
   FileMenu = XmCreateCascadeButton(MenuBar, "File", args, n);
   XtManageChild(FileMenu);

   n = 0;
   FileBar = XmCreatePulldownMenu(MenuBar, "File Pulldown", args, n);

   n = 0;
   XtSetArg(args[n], XmNsubMenuId, FileBar); n++;
   XtSetValues(FileMenu, args, n);

   n = 0;
   OpenButton = XtCreateManagedWidget(
      "Open...", xmPushButtonWidgetClass, FileBar, args, n);
   XtAddCallback(OpenButton, XmNactivateCallback, open_callback, "Open");
   
   n = 0;
   SaveClipButton = XtCreateManagedWidget(
      "Save Clip as...", xmPushButtonWidgetClass, FileBar, args, n);
   XtAddCallback(SaveClipButton, XmNactivateCallback, save_clip_callback, "SaveClipAs");
   
   n = 0;
   QuitButton = XtCreateManagedWidget(
      "Quit", xmPushButtonWidgetClass, FileBar, args, n);
   XtAddCallback(QuitButton, XmNactivateCallback, quit_callback, "Quit");
   

   Enlarge = XmCreatePushButton(Form, "Enlarge", NULL, 0);
   XtVaSetValues(Enlarge,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, MenuBar,
                 XmNleftAttachment, XmATTACH_FORM,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 25,
                 NULL);
   XtAddCallback(Enlarge, XmNactivateCallback, enlarge_call, NULL);
   XtManageChild(Enlarge);

   Reduce = XmCreatePushButton(Form, "Reduce", NULL, 0);
   XtVaSetValues(Reduce,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, MenuBar,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, Enlarge,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 50,
                 NULL);
   XtAddCallback(Reduce, XmNactivateCallback, reduce_call, NULL);
   XtManageChild(Reduce);

   ShiftLeft = XmCreatePushButton(Form, "Shift Left", NULL, 0);
   XtVaSetValues(ShiftLeft,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, MenuBar,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, Reduce,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 75,
                 NULL);
   XtAddCallback(ShiftLeft, XmNactivateCallback, shift_left_call, NULL);
   XtManageChild(ShiftLeft);

   ShiftRight = XmCreatePushButton(Form, "Shift Right", NULL, 0);
   XtVaSetValues(ShiftRight,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, MenuBar,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, ShiftLeft,
                 XmNrightAttachment, XmATTACH_FORM,
                 NULL);
   XtAddCallback(ShiftRight, XmNactivateCallback, shift_right_call, NULL);
   XtManageChild(ShiftRight);

   CutWidget = XmCreatePushButton(Form, "Cut", NULL, 0);
   XtVaSetValues(CutWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, Enlarge,
                 XmNleftAttachment, XmATTACH_FORM,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 25,
                 NULL);
   XtAddCallback(CutWidget, XmNactivateCallback, cut_call, NULL);
   XtManageChild(CutWidget);

   ResetWidget = XmCreatePushButton(Form, "Reset Markers", NULL, 0);
   XtVaSetValues(ResetWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, Enlarge,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, CutWidget,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 50,
                 NULL);
   XtAddCallback(ResetWidget, XmNactivateCallback, reset_call, NULL);
   XtManageChild(ResetWidget);

   EraseWidget = XmCreatePushButton(Form, "Erase Edits", NULL, 0);
   XtVaSetValues(EraseWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, Enlarge,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, ResetWidget,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 75,
                 NULL);
   XtAddCallback(EraseWidget, XmNactivateCallback, erase_call, NULL);
   XtManageChild(EraseWidget);

   PlayWidget = XmCreatePushButton(Form, "Play", NULL, 0);
   XtVaSetValues(PlayWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, CutWidget,
                 XmNleftAttachment, XmATTACH_FORM,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 25,
                 NULL);
   XtAddCallback(PlayWidget, XmNactivateCallback, play_call, NULL);
   XtManageChild(PlayWidget);

   PauseWidget = XmCreatePushButton(Form, "Pause", NULL, 0);
   XtVaSetValues(PauseWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, CutWidget,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, PlayWidget,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 50,
                 NULL);
   XtAddCallback(PauseWidget, XmNactivateCallback, pause_call, NULL);
   XtManageChild(PauseWidget);

   StopWidget = XmCreatePushButton(Form, "Stop", NULL, 0);
   XtVaSetValues(StopWidget,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, CutWidget,
                 XmNleftAttachment, XmATTACH_WIDGET,
                 XmNleftWidget, PauseWidget,
                 XmNrightAttachment, XmATTACH_POSITION,
                 XmNrightPosition, 75,
                 NULL);
   XtAddCallback(StopWidget, XmNactivateCallback, stop_call, NULL);
   XtManageChild(StopWidget);

   DrawArea = plot_build(Form, PlayWidget);

   XtManageChild(Form);

   XtRealizeWidget(toplevel);

   plot_create_gc();

   XtAppMainLoop(app);
}


