// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream.h>


#include "plot.h"

extern "C"
{
#include <X11/X.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/Xdbe.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
}

#include "aflibConfig.h"
#include "aflibAudioFile.h"
#include "aflibAudioEdit.h"
#include "aflibAudioSpectrum.h"
#include "aflibData.h"
#include "aflibSampleData.h"
#include "bstring.h"

#define DRAW_WIDTH 500
#define DRAW_HEIGHT 100
#define MARKER_DELTA 5
#define PLAY_DELTA 4096


extern void
expose_callback(
   Widget  w,
   caddr_t client_data,
   XmDrawingAreaCallbackStruct  *draw_struct);

static Widget DrawArea;
static GC draw_gc;
static XPoint  data_array[DRAW_WIDTH];
static int marker[3] = {0,DRAW_WIDTH-1, 0};
static int marker_selected = 0;
static XTextItem text_item[1];
static long start_sample, stop_sample;
static char start_string[100] = "00:00:00.00"; 
static char stop_string[100] = "00:00:00.00"; 
static char marker1_string[100] = "00:00:00.00";
static char marker2_string[100] = "00:00:00.00";
static long marker_samples_per_second = 0;
static int  marker_delta = 0;
static int divisor = 1;
static int shift_data = 0;
static aflibAudioFile * input = NULL;
static aflibAudioEdit * edit = NULL;
static aflibAudioFile * output = NULL;
static aflibAudioSpectrum * spec = NULL;
static unsigned long black_pixel = 0;
static unsigned long white_pixel = 0;
static unsigned long green_pixel = 0;
static unsigned long blue_pixel = 0;
static unsigned long red_pixel = 0;
static XtWorkProcId  work_id;
static long long current_position = 0;
static bool currently_playing = FALSE;

static XdbeBackBuffer back_buf = 0;
static XdbeSwapInfo   back_info;


void
button_press_callback(
   Widget w,
   XtPointer  data,
   XEvent    *event,
   Boolean   *flag)
{
   int x = event->xbutton.x;
   int d = MARKER_DELTA;

   if ((marker[0] > (x-d)) && (marker[0] < (x+d)))
   {
      marker_selected = 1;
      marker[0] = x;
   }
   if ((marker[1] > (x-d)) && (marker[1] < (x+d)))
   {
      marker_selected = 2;
      marker[1] = x;
   }
   expose_callback(DrawArea, NULL, NULL);
}

void
button_motion_callback(
   Widget w,
   XtPointer  data,
   XEvent    *event,
   Boolean   *flag)
{
   int x = event->xbutton.x;
   int hour, minutes;
   double seconds;

   if (marker_selected == 1)
   {
      marker[0] = x;
      seconds = (double)(x * marker_delta)/ marker_samples_per_second;
      hour = (int)(seconds / 3600);
      seconds -= (hour * 3600);
      minutes = (int)(seconds / 60);
      seconds -= (minutes * 60);
      sprintf(marker1_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);
   }
   else if (marker_selected == 2)
   {
      marker[1] = x;
      seconds = (double)(x * marker_delta)/ marker_samples_per_second;
      hour = (int)(seconds / 3600);
      seconds -= (hour * 3600);
      minutes = (int)(seconds / 60);
      seconds -= (minutes * 60);
      sprintf(marker2_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);
   }
   expose_callback(DrawArea, NULL, NULL);
}

void
button_release_callback(
   Widget  w,
   XtPointer  data,
   XEvent    *event,
   Boolean   *flag)
{
   marker_selected = 0;
   expose_callback(DrawArea, NULL, NULL);
}


void
plot_signal(double *db)
{
   Display *dis = XtDisplay(DrawArea);
   Window win = back_buf;

   //XClearArea(dis, XtWindow(DrawArea), DRAW_WIDTH, 0, 10, DRAW_HEIGHT + 140, FALSE);

   XSetForeground(dis, draw_gc, black_pixel);
   XFillRectangle(dis, win, draw_gc, DRAW_WIDTH, 0, 10, DRAW_HEIGHT + 140);
   XSetForeground(dis, draw_gc, red_pixel);

   int db_int = (int)db[0];

   db_int += 60;
   db_int *= 2;
   db_int = 120 - db_int;

   if (db_int > 120)
      db_int = 120;

   //cout << "Db is " << db << endl;

   XFillRectangle(dis, win, draw_gc, DRAW_WIDTH, db_int, 10, 120 - db_int);

   // We only swap in main expose routine to save CPU 
#if 0
   XdbeSwapBuffers(dis, &back_info, 1);
#endif

   XSetForeground(dis, draw_gc, white_pixel);
}

void
plot_spectrum(int array_size, double *db_array)
{
   int   i;

   Display *dis = XtDisplay(DrawArea);
   Window win = back_buf;

   //XClearArea(dis, XtWindow(DrawArea), 0, DRAW_HEIGHT + 20, DRAW_WIDTH, DRAW_HEIGHT + 140, FALSE);

   XSetForeground(dis, draw_gc, black_pixel);
   XFillRectangle(dis, win, draw_gc, 0, DRAW_HEIGHT + 20, array_size * 10, DRAW_HEIGHT + 140);
   XSetForeground(dis, draw_gc, red_pixel);

   // TBD I don't know why last element is bogus
   for (i = 0; i < array_size-1; i++)
   {
      int db_int = (int)(db_array[i]);

      db_int += 60;
      db_int *= (120/60);
      db_int = 120 - db_int;

      if (db_int > 120)
         db_int = 120;

      XFillRectangle(dis, win, draw_gc, i * 10, DRAW_HEIGHT + 20 + db_int, 9, DRAW_HEIGHT + 140);
   }

   // We only swap in main expose routine to save CPU 
#if 0
   XdbeSwapBuffers(dis, &back_info, 1);
#endif

   XSetForeground(dis, draw_gc, white_pixel);
}

void
expose_callback(
   Widget  w,
   caddr_t client_data,
   XmDrawingAreaCallbackStruct  *draw_struct)
{
   Display *dis = XtDisplay(w);
   Window win = back_buf;

   XSetForeground(dis, draw_gc, black_pixel);
   XFillRectangle(dis, win, draw_gc, 0, 0, DRAW_WIDTH, DRAW_HEIGHT + 30);
   XSetForeground(dis, draw_gc, white_pixel);

   XDrawLine(dis, win, draw_gc,
      0, DRAW_HEIGHT / 2,
      DRAW_WIDTH, DRAW_HEIGHT / 2);

   XDrawLines(dis, win, draw_gc, data_array, DRAW_WIDTH, CoordModeOrigin);

   text_item[0].font = None;
   text_item[0].delta = 0;
   text_item[0].nchars = strlen(start_string);
   text_item[0].chars = start_string;
   XDrawText(dis, win, draw_gc,
      0, DRAW_HEIGHT + 19, text_item, 1);

   text_item[0].nchars = strlen(stop_string);
   text_item[0].chars = stop_string;
   XDrawText(dis, win, draw_gc,
      DRAW_WIDTH - 70, DRAW_HEIGHT + 19, text_item, 1);

   XSetForeground(dis, draw_gc, green_pixel);
   text_item[0].nchars = strlen(marker1_string);
   text_item[0].chars = marker1_string;
   XDrawText(dis, win, draw_gc,
      DRAW_WIDTH / 2 - 80, DRAW_HEIGHT + 19, text_item, 1);
   /* Draw marker */
   XDrawLine(dis, win, draw_gc, marker[0], 0, marker[0], DRAW_HEIGHT);

   XSetForeground(dis, draw_gc, red_pixel);
   text_item[0].nchars = strlen(marker2_string);
   text_item[0].chars = marker2_string;
   XDrawText(dis, win, draw_gc,
      DRAW_WIDTH / 2 + 10, DRAW_HEIGHT + 19, text_item, 1);
   /* Draw marker */
   XDrawLine(dis, win, draw_gc, marker[1], 0, marker[1], DRAW_HEIGHT);

   if (output)
   {
      XSetForeground(dis, draw_gc, blue_pixel);
      /* Draw marker */
      XDrawLine(dis, win, draw_gc, marker[2], 0, marker[2], DRAW_HEIGHT);
   }

   XdbeSwapBuffers(dis, &back_info, 1);

   XSetForeground(dis, draw_gc, white_pixel);
}


void
plot_call(
   char * text_file)
{
   aflibConfig config;
   aflibStatus status = AFLIB_SUCCESS;
   static char * text_str = NULL;

   /* TBD This needs to be done better   */
   if (text_file != NULL)
   {
      if (text_str != NULL)
      {
         free(text_str);
      }
      text_str = strdup(text_file);

      delete edit;
      delete input;
      delete spec;

      input = new aflibAudioFile(AFLIB_WAV_TYPE, text_str, &config, &status);
      if (status != AFLIB_SUCCESS)
      {
         cout << "Error opening input device !" << endl;
      }
      spec = new aflibAudioSpectrum(*input);
      spec->setParameters(32, 5, 1);
      spec->setPowerMeterCallback(plot_signal);
      spec->setAudioSpectrumCallback(plot_spectrum);
      edit = new aflibAudioEdit(*spec);

      // Add the first segment which is the entire file
      config = input->getAudioConfig();
      edit->addSegment(1, 0, config.getTotalSamples(), 0);
   }

   config = edit->getAudioConfig();
   start_sample = 0;
   stop_sample = config.getTotalSamples() / divisor;
   int hour, minutes;
   double seconds;

   int i, array[DRAW_WIDTH];
   int ch_array[1];
   aflibSampleData sample(input);
   sample.setMaxMinMode(TRUE);
   sample.setOutputRange(0, DRAW_HEIGHT);
   sample.setNumSamplePoints(DRAW_WIDTH);
   sample.setParametersSamples(start_sample, stop_sample);
   ch_array[0] = 1;
   sample.setChannels(1, ch_array);
   sample.process(array);
   marker_delta = sample.getDelta();

   marker_samples_per_second = config.getSamplesPerSecond();
   seconds = (double)start_sample / config.getSamplesPerSecond();
   hour = (int)(seconds / 3600);
   seconds -= (hour * 3600);
   minutes = (int)(seconds / 60);
   seconds -= (minutes * 60);
   sprintf(start_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);

   seconds = (double)stop_sample / config.getSamplesPerSecond();
   hour = (int)(seconds / 3600);
   seconds -= (hour * 3600);
   minutes = (int)(seconds / 60);
   seconds -= (minutes * 60);
   sprintf(stop_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);

   seconds = (double)(marker[0] * marker_delta)/ marker_samples_per_second;
   hour = (int)(seconds / 3600);
   seconds -= (hour * 3600);
   minutes = (int)(seconds / 60);
   seconds -= (minutes * 60);
   sprintf(marker1_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);

   seconds = (double)(marker[1] * marker_delta)/ marker_samples_per_second;
   hour = (int)(seconds / 3600);
   seconds -= (hour * 3600);
   minutes = (int)(seconds / 60);
   seconds -= (minutes * 60);
   sprintf(marker2_string, "%2.2d:%2.2d:%05.2f", hour, minutes, seconds);

   for (i = 0; i < DRAW_WIDTH; i++)
   {
      data_array[i].y = array[i];
      data_array[i].x = i;
   }
   expose_callback(DrawArea, NULL, NULL);
}

void
plot_enlarge()
{
   divisor++;
   plot_call(NULL);
}

void
plot_reduce()
{
   divisor--;
   if (divisor == 0)
      divisor = 1;
   plot_call(NULL);
}

void
plot_shift_left()
{
   shift_data--;
}

void
plot_shift_right()
{
   shift_data++;
}

Boolean
plot_play_app(XtPointer client_data)
{
   aflibStatus status;

   currently_playing = TRUE;
   if (output == NULL)
   {
      current_position = 0;
      currently_playing = TRUE;
      return (TRUE);
   }

   if (current_position == 0)
   {
      current_position =  marker[0] * marker_delta;
   }

   // Update the play marker
   marker[2] = current_position / marker_delta;

   if (current_position >= marker[1] * marker_delta)
   {
      XtRemoveWorkProc(work_id);
      delete output;
      output = NULL;
      currently_playing = TRUE;
      current_position = 0;
   }
   else
   {
      output->process(status, current_position, PLAY_DELTA);

      current_position += PLAY_DELTA;
   }

  expose_callback(DrawArea, NULL, NULL);
   return(FALSE);
}

void
plot_play(char * text_file)
{
   aflibConfig config;
   aflibStatus status = AFLIB_SUCCESS;
   Display *dis = XtDisplay(DrawArea);
   XtAppContext app = XtDisplayToApplicationContext(dis);

   work_id = XtAppAddWorkProc(app, plot_play_app, NULL);

   delete output;
   config = input->getConfig();

   if (text_file == NULL)
   {
      output = new aflibAudioFile(*edit, AFLIB_DEV_TYPE, "/dev/audio", &config, &status);
   }
   else
   {
      output = new aflibAudioFile(*edit, AFLIB_WAV_TYPE, text_file, &config, &status);
   }
   if (status != AFLIB_SUCCESS)
   {
      cout << "Error opening output device !" << endl;
   }
}

void
plot_pause()
{
   Display *dis = XtDisplay(DrawArea);
   XtAppContext app = XtDisplayToApplicationContext(dis);

   if (currently_playing == TRUE)
   {
      XtRemoveWorkProc(work_id);
      currently_playing = FALSE;
   }
   else
   {
      work_id = XtAppAddWorkProc(app, plot_play_app, NULL);
      currently_playing = TRUE;
   }
}

void
plot_stop()
{
   XtRemoveWorkProc(work_id);
   delete output;
   output = NULL;
   currently_playing = TRUE;
   current_position = 0;
   expose_callback(DrawArea, NULL, NULL);
}

void
plot_cut()
{
   edit->removeSegment((long long)marker[0] * marker_delta, (long long)marker[1] * marker_delta);

   plot_call(NULL);
}

void
plot_reset()
{
   marker[0] = 0;
   marker[1] = DRAW_WIDTH-1;

   plot_call(NULL);
   expose_callback(DrawArea, NULL, NULL);
}

void
plot_erase()
{
   while (edit->getNumberOfSegments())
   {
      edit->removeSegment(1);
   }

   aflibConfig config = input->getAudioConfig();
   edit->addSegment(1, 0, config.getTotalSamples(), 0);

   plot_call(NULL);
}

void
plot_create_gc()
{
   /* Need to do this after toplevel is realized */
   XGCValues gcv;
   gcv.foreground = white_pixel;
   gcv.background = black_pixel;

   // Use double buffering for smooth display
   if (back_buf == 0)
   {
      XdbeBackBufferAttributes *attr;

      back_buf = XdbeAllocateBackBufferName(XtDisplay(DrawArea), XtWindow(DrawArea), XdbeUndefined);
      attr = XdbeGetBackBufferAttributes(XtDisplay(DrawArea), back_buf);
      back_info.swap_window = attr->window;
      back_info.swap_action = XdbeUndefined;
   }

   draw_gc = XCreateGC (XtDisplay(DrawArea), back_buf,
      GCForeground | GCBackground, &gcv);

}


Widget
plot_build(
   Widget parent,
   Widget top)
{
   Arg  args[20];
   int  n;
   XColor  red, green, blue;
   Display *dpy;

   dpy = XtDisplay(parent);

   black_pixel = BlackPixelOfScreen(XtScreen(parent));
   white_pixel = WhitePixelOfScreen(XtScreen(parent));

   red.red = 65535;
   red.green = 0;
   red.blue = 0;
   XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &red);
   red_pixel = red.pixel;

   green.red = 0;
   green.green = 65535;
   green.blue = 0;
   XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &green);
   green_pixel = green.pixel;

   blue.red = 0;
   blue.green = 0;
   blue.blue = 65535;
   XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &blue);
   blue_pixel = blue.pixel;

   n = 0;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, top); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNwidth, DRAW_WIDTH + 10); n++;
   XtSetArg(args[n], XmNheight, DRAW_HEIGHT + 140); n++;
   XtSetArg(args[n], XmNbackground, BlackPixelOfScreen(XtScreen(parent))); n++;
   DrawArea = XmCreateDrawingArea(parent, "Draw", args, n);
   XtAddCallback(DrawArea, XmNexposeCallback, expose_callback, NULL);
   XtAddEventHandler(
      DrawArea, ButtonPressMask, FALSE, button_press_callback, NULL);
   XtAddEventHandler(
      DrawArea, ButtonMotionMask, FALSE, button_motion_callback, NULL);
   XtAddEventHandler(
      DrawArea, ButtonReleaseMask, FALSE, button_release_callback, NULL);
   XtManageChild(DrawArea);


   return (DrawArea);
}


