/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


// class for MP3 audio file reading and writing

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>

#include "aflibMp3File.h"
#include "aflibData.h"


aflibMp3File::aflibMp3File()
{
   _fd = 0;
   _data_start = 0;
   _data_length = 0;
   _length_seek = 0;
   _length_value = 0;
}

aflibMp3File::~aflibMp3File()
{
   // Should be moved to a close function
   if (_fd != 0)
   {
      pclose(_fd);
   }
}

aflibStatus
aflibMp3File::afopen(
   const string& file,
   aflibConfig* cfg)
{
   // This function needs to be redone to find the header info correctly
   // This function will open an existing MP3 file.

   aflibStatus  status = AFLIB_SUCCESS;
   //string cmd_str("mpg123 -q -s ");
   string cmd_str("mpg123 -s ");
   FILE *file_fd;
   unsigned long newhead;
   unsigned char hbuf[10];
   int lsf, sampling_frequency, mpeg25;


   file_fd = fopen(file.c_str(), "r");
   if (file_fd == NULL)
   {
      status = AFLIB_ERROR_OPEN;
      return (status);
   }

   fread(hbuf, 4, 1, file_fd);
   fclose(file_fd);

   _mpeg_header.h_bytes.bytes0 = hbuf[0];
   _mpeg_header.h_bytes.bytes1 = hbuf[1];
   _mpeg_header.h_bytes.bytes2 = hbuf[2];
   _mpeg_header.h_bytes.bytes3 = hbuf[3];

#if 0
   cout << "Byte 0 is " << hex << (int)_mpeg_header.h_bytes.bytes0 << endl;
   cout << "Byte 1 is " << (int)_mpeg_header.h_bytes.bytes1 << endl;
   cout << "Byte 2 is " << (int)_mpeg_header.h_bytes.bytes2 << endl;
   cout << "Byte 3 is " << (int)_mpeg_header.h_bytes.bytes3 << endl;
#endif
   
   cout << "Header is " << (unsigned int)_mpeg_header.h_mpeg.syncword << endl;
   cout << "Layer played is " << _mpeg_header.h_mpeg.layer << endl;
   cout << "ID is " << _mpeg_header.h_mpeg.id << endl;
   cout << "Error Protection is " << _mpeg_header.h_mpeg.error_prot << endl;
   cout << "Bit Rate Index is " << _mpeg_header.h_mpeg.bitrate_index << endl;
   cout << "Sampling Frequency is " << _mpeg_header.h_mpeg.sampl_freq << endl;
   cout << "Padding Bit is " << _mpeg_header.h_mpeg.pad << endl;
   cout << "Private Bit is " << _mpeg_header.h_mpeg.private_bit << endl;
   cout << "Mode is " << _mpeg_header.h_mpeg.mode << endl;
   cout << "Mode Extension is " << _mpeg_header.h_mpeg.mode_ext << endl;
   cout << "Copyright is " << _mpeg_header.h_mpeg.copyright << endl;
   cout << "Original is " << _mpeg_header.h_mpeg.original << endl;
   cout << "Emphasis is " << _mpeg_header.h_mpeg.emphasis << endl;

   newhead = ((unsigned long) hbuf[0] << 24) | ((unsigned long) hbuf[1] << 16) |
            ((unsigned long) hbuf[2] << 8) | (unsigned long) hbuf[3];

   if( newhead & (1<<20) )
   {
      lsf = (newhead & (1<<19)) ? 0x0 : 0x1;
      mpeg25 = 0;
   }
   else
   {
      lsf = 1;
      mpeg25 = 1;
   }
   if(mpeg25)
      sampling_frequency = 6;
   else
      sampling_frequency = ((newhead>>10)&0x3) + (lsf*3);

   cmd_str.append(file.c_str());

   _fd = popen(cmd_str.c_str(), "r");
   if (_fd == NULL)
   {
      status = AFLIB_ERROR_OPEN;
      return (status);
   }

   _cfg.setChannels(2);
   _cfg.setSampleSize(AFLIB_DATA_16S);
   if (sampling_frequency == 3)
      _cfg.setSamplesPerSecond(22050);
   // I think this should be 6. Don't know why need to check mpeg header
   else if (sampling_frequency == 0)
      _cfg.setSamplesPerSecond(44100);
   else
   {
      status = AFLIB_ERROR_UNSUPPORTED;
      return (status);
   }
   _cfg.setDataOrientation(AFLIB_INTERLEAVE_LE);

   return (status);
}

aflibStatus
aflibMp3File::afcreate(
   const string& file,
   const aflibConfig& cfg)
{
   aflibStatus  status = AFLIB_SUCCESS;
   unsigned char  buf[100];
   int block_align;
   int avg_bytes;
   long num_samples;
   string cmd_str("encode ");

   if (cfg.getDataQuality() == AFLIB_QUALITY_LOW)
   {
      cmd_str.append("-l 2 -p 1 -b 32 ");
   }
   else if (cfg.getDataQuality() == AFLIB_QUALITY_MED)
   {
      cmd_str.append("-l 2 -p 1 -b 48 ");
   }
   else if ((cfg.getDataQuality() == AFLIB_QUALITY_HIGH) &&
           (cfg.getSamplesPerSecond() == 22050))
   {
      cmd_str.append("-l 2 -p 1 -b 128 ");
   }
   else if (cfg.getDataQuality() == AFLIB_QUALITY_HIGH)
   {
      cmd_str.append("-l 3 -p 2 -b 128 -q 99.9 ");
   }

   // Select stereo or mono
   if (cfg.getChannels() == 2)
   {
      cmd_str.append("-m s ");
   }
   else if (cfg.getChannels() == 1)
   {
      cmd_str.append("-m m ");
   }
   else
   {
      status = AFLIB_ERROR_UNSUPPORTED;
      return(status);
   }

   // Select samples per second
   if (cfg.getSamplesPerSecond() == 44100)
   {
      cmd_str.append("-s 44.1 ");
   }
   else if (cfg.getSamplesPerSecond() == 22050)
   {
      cmd_str.append("-s 22.05 ");
   }
   else
   {
      status = AFLIB_ERROR_UNSUPPORTED;
      return(status);
   }

   cmd_str.append("-L - ");
   cmd_str.append(file.c_str());

   //cmd_str.append(" > /dev/null");

   cout << cmd_str.c_str() << endl;

   _fd = popen(cmd_str.c_str(), "w");
   if (_fd == NULL)
   {
      status = AFLIB_ERROR_OPEN;
      return (status);
   }

   fwrite("RIFF WAVE ", 10, 1, _fd);

   buf[0] = 'f';
   buf[1] = 'm';
   buf[2] = 't';
   buf[3] = ' ';

   // length will be stored at close
   _length_seek = 14;
   buf[4] = 0;
   buf[5] = 0;
   buf[6] = 0;
   buf[7] = 0;

   // format
   buf[8] = 1;
   buf[9] = 0;

   // # of channels
   buf[10] = cfg.getChannels();
   buf[11] = 0;

   num_samples = cfg.getSamplesPerSecond();
   // # of samples
   buf[15] = num_samples / (256 * 256 * 256);
   num_samples -= (256 * 256 * 256 * buf[15]);
   buf[14] = num_samples / (256 * 256);
   num_samples -= (256 * 256 * buf[14]);
   buf[13] = num_samples / (256);
   num_samples -= (256 * buf[13]);
   buf[12] = num_samples;

   block_align = cfg.getChannels() * cfg.getBitsPerSample() / 8;
   avg_bytes = block_align * cfg.getSamplesPerSecond();
   // Avg # of bytes
   buf[19] = avg_bytes / (256 * 256 * 256);
   avg_bytes = avg_bytes - (256 * 256 * 256 * buf[19]);
   buf[18] = avg_bytes / (256 * 256);
   avg_bytes = avg_bytes - (256 * 256 * buf[18]);
   buf[17] = avg_bytes / (256);
   avg_bytes = avg_bytes - (256 * buf[17]);
   buf[16] = avg_bytes;

   // Block Align
   buf[20] = (unsigned char)block_align;
   buf[21] = 0;

   // Bits per Sample
   buf[22] = cfg.getBitsPerSample();
   buf[23] = 0;

   fwrite(buf, 24, 1, _fd);

   return (status);
}

aflibStatus
aflibMp3File::afread(
   aflibData& data,
   long long position )
{
   // Read data from a wave file. A -1 for position means read from the
   // current position.

   void * p_data;
   long   total_length;
   long   new_length;
   aflibStatus  status = AFLIB_SUCCESS;

   data.setConfig(_cfg);
   total_length = data.getTotalLength();
   p_data = data.getDataPointer();

   new_length = fread(p_data, total_length, 1, _fd);
   if (new_length != total_length)
   {
      //TBD may need some check
   }
   if (new_length == 0)
   {
      status = AFLIB_END_OF_FILE;
   }

   return(status);
}

aflibStatus
aflibMp3File::afwrite(
   aflibData& data,
   long long position )
{
   void * p_data;
   long   total_length;
   aflibStatus  status = AFLIB_SUCCESS;
   aflibConfig _config;
   int    i;
   unsigned char  zero = 0;
   int temp;
   signed char temp_char;


   _config = data.getConfig();
   total_length = data.getTotalLength();
   p_data = data.getDataPointer();

   if (_config.getBitsPerSample() == 8)
   {
      for (i = 0; i < total_length; i++)
      {
         fwrite(&zero, 1, 1, _fd);
         temp = *((unsigned char *)p_data + i);
         temp = temp - 127;
         temp_char = (signed char)temp;
         fwrite(&temp_char, 1, 1, _fd);
      }
   }
   else
   {
      fwrite(p_data, total_length, 1, _fd);
   }

   _length_value += total_length;

   return (status);
}



