/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */


// Base class for MP3 audio file reading and writing

#ifndef _AFLIBMP3FILE_H
#define _AFLIBMP3FILE_H

#include "aflibFile.h"
#include "bstring.h"

class aflibMp3File : public aflibFile {

public:

   aflibMp3File();

   ~aflibMp3File();

   aflibStatus
   afopen(
      const string& file,
      aflibConfig* cfg);

   aflibStatus
   afcreate(
      const string& file,
      const aflibConfig& cfg);

   aflibStatus
   afread(
      aflibData& data,
      long long position = -1);

   aflibStatus
   afwrite(
      aflibData& data,
      long long position = -1);

   bool
   isSizeSupported(
      aflib_data_size data_size)
   { return (data_size == AFLIB_DATA_16S);};

   bool
   isEndianSupported(
      aflib_data_endian data_end)
      { return (data_end == AFLIB_ENDIAN_LITTLE);};


private:

struct  header_bytes
{
   unsigned char bytes3;
   unsigned char bytes2;
   unsigned char bytes1;
   unsigned char bytes0;
};

struct mpeg_header
{
   unsigned int emphasis:2;
   unsigned int original:1;
   unsigned int copyright:1;
   unsigned int mode_ext:2;
   unsigned int mode:2;
   unsigned int private_bit:1;
   unsigned int pad:1;
   unsigned int sampl_freq:2;
   unsigned int bitrate_index:4;
   unsigned int error_prot:1;
   unsigned int layer:2;
   unsigned int id:1;
   unsigned int syncword:12;
};

union mpeg_union
{
   header_bytes h_bytes;
   mpeg_header  h_mpeg;
};


FILE*    _fd;
long     _data_start;
long     _data_length;
long     _length_seek;
long     _length_value;
mpeg_union _mpeg_header;

};


#endif
