/*
 * Copyright: (C) 1999 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */

// class for WAVE audio file reading and writing

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream.h>

#include "aflibWavFile.h"
#include "aflibData.h"


aflibWavFile::aflibWavFile()
{
   _fd_int = -1;
   _riff_length = -4;
   _data_length = -4;
}

aflibWavFile::~aflibWavFile()
{
   if (_fd_int != -1)
   {
      close(_fd_int);
   }
}

aflibStatus
aflibWavFile::afopen(
   const string& file,
   aflibConfig* cfg)
{
   // This function needs to be redone to find the header info correctly
   // This function will open an existing WAV file.

   aflibStatus  status = AFLIB_SUCCESS;
   unsigned char  buf[10000];
   int i;
   long long length;

   _fd_int = ::open(file.c_str(), O_RDONLY, 0);
   if (_fd_int < 0)
   {
      status = AFLIB_ERROR_OPEN;
      return (status);
   }

   // Store user specified results as defaults
   if (cfg != NULL)
   {
      _cfg = *cfg;
   }

   // Read the header to identify file
   read(_fd_int, buf, 9999);

   if (buf[0] == 'R' && buf[1] == 'I' && buf[2] == 'F' && buf[3] == 'F')
   {
      length = (long long)buf[4] + buf[5] * 256 + buf[6] * 256 * 256 +
                        buf[7] * 256 * 256 * 256;
      for (i = 0; i < 10000; i++)
      {
         if (buf[i] == 'f')
         {
            if (buf[i+1] == 'm' && buf[i+2] == 't' && buf[i+3] == ' ')
            {
               int format;
               long sam;
               long avg_bytes;
               int align;
               int bits;
               int channels;

               cout << "fmt field found!" << endl;
               format = buf[i+8] | (buf[i+9] << 8);
               channels = buf[i+10] | (buf[i+11] << 8);
               sam = buf[i+12] + buf[i+13] * 256 + buf[i+14] * 256 * 256 +
                     buf[i+15] * 256 * 256 * 256;
               avg_bytes=buf[i+16] + buf[i+17] * 256 + buf[i+18] * 256 * 256 +
                     buf[i+19] * 256 * 256 * 256;
               align = buf[i+20] | (buf[i+21] << 8);
               bits = buf[i+22] | (buf[i+23] << 8);

               // Set the time statistics 
               _total_size = length;
               _current_place_size = 0;
               _one_second_size = avg_bytes;

               _cfg.setTotalSamples(length / (avg_bytes/sam)); 
               _cfg.setChannels(channels);

               // Need to set this better
               if (bits == 8)
               {
                  _cfg.setSampleSize(AFLIB_DATA_8U);
               }
               else if (bits == 16)
               {
                  _cfg.setSampleSize(AFLIB_DATA_16S);
               }
               _cfg.setSamplesPerSecond(sam);
               _cfg.setDataOrientation(AFLIB_INTERLEAVE_LE);
 
               cout << "Length is " << length << endl;
               cout << "Format is " << format << " Channels " << channels << endl;
               cout << "Samples per second are : " << sam << endl;
               cout << "Avg Bytes per second are : " << avg_bytes << endl;
               cout << "Block Align " << align << endl;
               cout << "Bits per Sample " << bits << endl;
            }
         }

         if (buf[i] == 'd')
         {
            if (buf[i+1] == 'a' && buf[i+2] == 't' && buf[i+3] == 'a')
            {
               long length;

               length = buf[i+4] + buf[i+5] * 256 + buf[i+6] * 256 * 256 +
                        buf[i+7] * 256 * 256 * 256;

               cout << "data field found!" << endl;
               cout << "Length is " << length << endl;

               _data_start = i + 8;
               break;
            }
         }
      }
      // reset to start
      lseek(_fd_int, _data_start, SEEK_SET);
   }
   else
   {
      status = AFLIB_ERROR_UNSUPPORTED;
      return (status);
   }
   return (status);
}

aflibStatus
aflibWavFile::afcreate(
   const string& file,
   const aflibConfig& cfg)
{
   aflibStatus  status = AFLIB_SUCCESS;
   unsigned char  buf[100];
   int block_align;
   int avg_bytes;
   long num_samples;

   _fd_int = ::open(file.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0664);
   if (_fd_int < 0)
   {
      status = AFLIB_ERROR_OPEN;
      return (status);
   }

   write(_fd_int, "RIFF    WAVE", 12);

   _riff_length_offset = 4;

   buf[0] = 'f';
   buf[1] = 'm';
   buf[2] = 't';
   buf[3] = ' ';

   buf[4] = 16;
   buf[5] = 0;
   buf[6] = 0;
   buf[7] = 0;

   // format
   buf[8] = 1;
   buf[9] = 0;

   // # of channels
   buf[10] = cfg.getChannels();
   buf[11] = 0;

   num_samples = cfg.getSamplesPerSecond();
   // # of samples
   buf[15] = num_samples / (256 * 256 * 256);
   num_samples -= (256 * 256 * 256 * buf[15]);
   buf[14] = num_samples / (256 * 256);
   num_samples -= (256 * 256 * buf[14]);
   buf[13] = num_samples / (256);
   num_samples -= (256 * buf[13]);
   buf[12] = num_samples;

   block_align = cfg.getChannels() * cfg.getBitsPerSample() / 8;
   avg_bytes = block_align * cfg.getSamplesPerSecond();
   // Avg # of bytes
   buf[19] = avg_bytes / (256 * 256 * 256);
   avg_bytes = avg_bytes - (256 * 256 * 256 * buf[19]);
   buf[18] = avg_bytes / (256 * 256);
   avg_bytes = avg_bytes - (256 * 256 * buf[18]);
   buf[17] = avg_bytes / (256);
   avg_bytes = avg_bytes - (256 * buf[17]);
   buf[16] = avg_bytes;

   // Block Align
   buf[20] = (unsigned char)block_align;
   buf[21] = 0;

   // Bits per Sample
   buf[22] = cfg.getBitsPerSample();
   buf[23] = 0;

   buf[24] = 'd';
   buf[25] = 'a';
   buf[26] = 't';
   buf[27] = 'a';

   _data_length_offset = 40;

   write(_fd_int, buf, 28);

   return(status);
}

aflibStatus
aflibWavFile::afread(
   aflibData& data,
   long long position )
{
   // Read data from a wave file. A -1 for position means read from the
   // current position. Position should be specified as samples. It starts
   // at position 1. 

   void * p_data;
   long   total_length;
   long   new_length;
   aflibStatus  status = AFLIB_SUCCESS;

   if (position != -1)
   {
      lseek(_fd_int, _data_start +
         position * _cfg.getChannels() * _cfg.getBitsPerSample() / 8,
         SEEK_SET);
   }

   data.setConfig(_cfg);
   total_length = data.getTotalLength();
   p_data = data.getDataPointer();

   new_length = read(_fd_int, p_data, total_length);
   if (new_length != total_length)
   {
      //TBD may need some check
   }
   if (new_length == 0)
   {
      status = AFLIB_END_OF_FILE;
   }

   _current_place_size += new_length;
//   cout << "Current position is " << _current_place_size / _one_second_size << " of a total " << _total_size / _one_second_size << endl;

   return(status);
}

aflibStatus
aflibWavFile::afwrite(
   aflibData& data,
   long long position )
{
   void * p_data;
   long   total_length;
   aflibStatus  status = AFLIB_SUCCESS;
   long tmp_length_value;
   unsigned char buf[10];
   long cur_pos;

   total_length = data.getTotalLength();
   p_data = data.getDataPointer();

   write(_fd_int, p_data, total_length);

   _data_length += total_length;
   _riff_length = _data_length + 36;

   tmp_length_value = _riff_length;
   buf[3] = tmp_length_value / (256 * 256 * 256);
   tmp_length_value -= (256 * 256 * 256 * buf[3]);
   buf[2] = tmp_length_value / (256 * 256);
   tmp_length_value -= (256 * 256 * buf[2]);
   buf[1] = tmp_length_value / (256);
   tmp_length_value -= (256 * buf[1]);
   buf[0] = tmp_length_value;
   cur_pos = lseek(_fd_int, 0, SEEK_CUR);
   lseek(_fd_int, _riff_length_offset, SEEK_SET);
   write(_fd_int, buf, 4);

   tmp_length_value = _data_length;
   buf[3] = tmp_length_value / (256 * 256 * 256);
   tmp_length_value -= (256 * 256 * 256 * buf[3]);
   buf[2] = tmp_length_value / (256 * 256);
   tmp_length_value -= (256 * 256 * buf[2]);
   buf[1] = tmp_length_value / (256);
   tmp_length_value -= (256 * buf[1]);
   buf[0] = tmp_length_value;
   lseek(_fd_int, _data_length_offset, SEEK_SET);
   write(_fd_int, buf, 4);

   lseek(_fd_int, cur_pos, SEEK_SET);

   return(status);
}


