// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <X11/X.h>

extern "C"
{
#include <Xm/Xm.h>
}

#include "gui.h"
#include "audio.h"

#include "aflibConfig.h"
#include "aflibAudioFile.h"
#include "aflibAudioSpectrum.h"
#include "aflibData.h"
#include "aflibSampleData.h"
#include "bstring.h"


#define PLAY_DELTA 2048

static bool play = FALSE;
static aflibAudio  *input = NULL;
static aflibAudioSpectrum  *spec = NULL;
static aflibAudio  *output = NULL;
static aflibConfig     input_config;
static long long  position = 0;
static long long  total_size = 0;
static long long  samples_per_second = 0;
static string play_file;
static string device_file;

Boolean
work_app(XtPointer client_data)
{
   aflibStatus  status;

   if (play == TRUE)
   {
      output->process(status, position, PLAY_DELTA);
      if (status != AFLIB_SUCCESS)
      {
         audio_stop();
      }
      position += PLAY_DELTA;
      gui_set_slider_position(position, total_size, samples_per_second);
   }
   else
   {
      usleep(1);
   }

   return FALSE;
}

void
audio_spectrum(int array_size, double *db_array)
{
   gui_spectrum(array_size, db_array);
}

void
audio_play(char * _play)
{
   aflibStatus status;

   if (_play != NULL)
   {
      play_file = _play;
   }

   // If no chain then create
   if (output == NULL)
   {
      if (strcmp(play_file.c_str() + play_file.length() - 4, ".wav") == 0)
      {
         input = new aflibAudioFile(AFLIB_WAV_TYPE, play_file.c_str(),
            &input_config, &status);
      }
      else
      {
         input = new aflibAudioFile(AFLIB_MPEG_TYPE, play_file.c_str(),
            &input_config, &status);
      }

      total_size = input_config.getTotalSamples();
      samples_per_second = input_config.getSamplesPerSecond();

      spec = new aflibAudioSpectrum(*input);
      spec->setParameters(32, 5, 2);
      spec->setAudioSpectrumCallback(audio_spectrum);

      output = new aflibAudioFile(*spec, AFLIB_DEV_TYPE, device_file.c_str(),
         &input_config, &status);
   }
   play = TRUE;
}

void
audio_record(
   const char * _play,
   aflibConfig& config,
   const char * format)
{
   aflibStatus status;

   if (_play != NULL)
   {
      play_file = _play;
   }

   // If no chain then create
   if (output == NULL)
   {
      input_config = config;
      input = new aflibAudioFile(AFLIB_DEV_TYPE, device_file.c_str(),
         &input_config, &status);

      total_size = input_config.getTotalSamples();
      samples_per_second = input_config.getSamplesPerSecond();

      spec = new aflibAudioSpectrum(*input);
      spec->setParameters(32, 5, 2);
      spec->setAudioSpectrumCallback(audio_spectrum);

      if (strcmp("WAV", format) == 0)
      {
         output = new aflibAudioFile(*spec, AFLIB_WAV_TYPE, play_file.c_str(),
            &input_config, &status);
      }
      else if (strcmp("MP3", format) == 0)
      {
         output = new aflibAudioFile(*spec, AFLIB_MPEG_TYPE, play_file.c_str(),
            &input_config, &status);
      }
      else
      {
         cerr << "SOFTWARE ERROR!" << endl;
      }
   }
   play = TRUE;
}

void
audio_pause()
{
   play = FALSE;
}

void
audio_stop()
{
   play = FALSE;
   position = 0;

   delete output;
   output = NULL;

   delete spec;
   spec = NULL;

   delete input;
   input = NULL;

   // If recording turn off
   gui_set_record_mode(FALSE);

}

void
audio_device( const char * device )
{
   device_file = device;
}

void
audio_set_position(int pos)
{
   position = (pos * total_size) / 100;
}



