// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include <X11/X.h>

extern "C"
{
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/DrawingA.h>
#include <Xm/Scale.h>
#include <Xm/RowColumn.h>
}

#include "pixmaps.h"

#include "aflibConfig.h"
#include "aflibAudioFile.h"
#include "aflibData.h"
#include "aflibSampleData.h"
#include "aflibEnvFile.h"
#include "bstring.h"

#include "lbdraw.h"
#include "rbdraw.h"
#include "rtdraw.h"
#include "audio.h"
#include "popup.h"


extern void
expose_callback(
   Widget  w,
   caddr_t client_data,
   XmDrawingAreaCallbackStruct  *draw_struct);

extern void
scale_callback(
   Widget  w,
   caddr_t client_data,
   XmScaleCallbackStruct *scale_struct);


extern void
gui_postit(
   Widget pb,
   void *popup,
   XEvent  *event,
   Boolean              *flag);


static XrmOptionDescRec opTable[] =
{
   {"-time", ".time", XrmoptionSepArg, NULL},
};

static String FallBack[] =
{
   "*borderWidth: 0",
   "*geometry: +100+100",
   "*background: grey",
   NULL
};

static lbdraw  * _lbdraw = NULL;
static rbdraw  * _rbdraw = NULL;
static rtdraw  * _rtdraw = NULL;
static Widget PositionScale;;


void
stop_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   audio_stop();
}

void
play_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   audio_play(NULL);
}

void
pause_call(
   Widget w,
   XtPointer client_data,
   XtPointer call_data)
{
   audio_pause();
}

void
gui_spectrum(
   int array,
   double *db_array)
{
   _rbdraw->plot_spectrum(array, db_array);
   _rtdraw->plot_spectrum(array, &db_array[32]);
}

void
gui_build(int argc, char * argv[])
{
   Widget toplevel, Form;
   Widget StopButton, PlayButton, PauseButton;
   Widget StatusText;
   Widget LTDrawArea;
   Widget popup;
   Pixmap stop_pixmap, pause_pixmap, play_pixmap;
   Pixel  fg, bg;
   XtAppContext  app;
   Arg  args[20];
   int  n;


   toplevel = XtVaAppInitialize(&app, "Form1",
               opTable, XtNumber(opTable),
               &argc, argv,
               FallBack, NULL);

   Form = XmCreateForm(toplevel, "Form", NULL, 0);
   XtVaSetValues(Form,
                 XmNresizePolicy, XmRESIZE_ANY,
                 XmNallowShellResize, True,
                 NULL);

   popup = popup_build(Form, XmMENU_POPUP);

   XtVaGetValues(Form,
      XmNbackground, &bg,
      XmNforeground, &fg,
      NULL);

   stop_pixmap = XCreatePixmapFromBitmapData(XtDisplay(toplevel),
      RootWindowOfScreen(XtScreen(toplevel)),
      (char *)stop_pixmap_bits, stop_pixmap_width, stop_pixmap_height, fg, bg, DefaultDepthOfScreen(XtScreen(toplevel)));

   play_pixmap = XCreatePixmapFromBitmapData(XtDisplay(toplevel),
      RootWindowOfScreen(XtScreen(toplevel)),
      (char *)play_pixmap_bits, play_pixmap_width, play_pixmap_height, fg, bg, DefaultDepthOfScreen(XtScreen(toplevel)));

   pause_pixmap = XCreatePixmapFromBitmapData(XtDisplay(toplevel),
      RootWindowOfScreen(XtScreen(toplevel)),
      (char *)pause_pixmap_bits, pause_pixmap_width, pause_pixmap_height, fg, bg, DefaultDepthOfScreen(XtScreen(toplevel)));

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
   XtSetArg(args[n], XmNlabelPixmap, stop_pixmap); n++;
   StopButton = XmCreatePushButton(Form, "Stop Button", args, n);
   XtAddCallback(StopButton, XmNactivateCallback, stop_call, NULL);
   XtManageChild(StopButton);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, StopButton); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
   XtSetArg(args[n], XmNlabelPixmap, play_pixmap); n++;
   PlayButton = XmCreatePushButton(Form, "Play Button", args, n);
   XtAddCallback(PlayButton, XmNactivateCallback, play_call, NULL);
   XtManageChild(PlayButton);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, PlayButton); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
   XtSetArg(args[n], XmNlabelPixmap, pause_pixmap); n++;
   PauseButton = XmCreatePushButton(Form, "Pause Button", args, n);
   XtAddCallback(PauseButton, XmNactivateCallback, pause_call, NULL);
   XtManageChild(PauseButton);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, PauseButton); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNeditMode, XmSINGLE_LINE_EDIT); n++;
   XtSetArg(args[n], XmNeditable, FALSE); n++;
   XtSetArg(args[n], XmNwidth, 100); n++;
   XtSetArg(args[n], XmNautoShowCursorPosition, FALSE); n++;
   XtSetArg(args[n], XmNcursorPositionVisible, FALSE); n++;
   StatusText = XmCreateText(Form, "Status Text", args, n);
   XtManageChild(StatusText);

   n = 0; 
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNbottomWidget, StatusText); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNmaximum, 100); n++;
   XtSetArg(args[n], XmNminimum, 0); n++;
   XtSetArg(args[n], XmNvalue, 0); n++;
   XtSetArg(args[n], XmNshowValue, FALSE); n++;
   XtSetArg(args[n], XmNprocessingDirection, XmMAX_ON_RIGHT); n++;
   XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
   XtSetArg(args[n], XmNscaleWidth, 300); n++;
   PositionScale = XmCreateScale(Form, "Position Scale", args, n);
   XtAddCallback(PositionScale, XmNdragCallback, scale_callback, NULL);
   XtManageChild(PositionScale);

   _lbdraw = new lbdraw(Form, PositionScale);
   XtAddEventHandler(_lbdraw->getWidget(), ButtonPressMask, False, gui_postit, popup);

   _rbdraw = new rbdraw(Form, _lbdraw->getWidget(), PositionScale);
   XtAddEventHandler(_rbdraw->getWidget(), ButtonPressMask, False, gui_postit, popup);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNbottomWidget, _lbdraw->getWidget()); n++;
   XtSetArg(args[n], XmNwidth, 150); n++;
   XtSetArg(args[n], XmNheight, 60); n++;
   XtSetArg(args[n], XmNbackground, BlackPixelOfScreen(XtScreen(Form))); n++;
   LTDrawArea = XmCreateDrawingArea(Form, "Draw LT", args, n);
   XtAddCallback(LTDrawArea, XmNexposeCallback, expose_callback, NULL);
   XtManageChild(LTDrawArea);
   XtAddEventHandler(LTDrawArea, ButtonPressMask, False, gui_postit, popup);

   _rtdraw = new rtdraw(Form, LTDrawArea, _rbdraw->getWidget());
   XtAddEventHandler(_rtdraw->getWidget(), ButtonPressMask, False, gui_postit, popup);

   XtManageChild(Form);

   XtRealizeWidget(toplevel);

   _lbdraw->after_realize();
   _rbdraw->after_realize();
   _rtdraw->after_realize();

   _lbdraw->setTime(0);
   _lbdraw->setRecordMode(FALSE);

   XtAppAddWorkProc(app, work_app, NULL);

   // Get device port or set if not found
   aflibEnvFile env(ENV_FILE);
   string key(PORT_KEY);
   string value;
   if (env.readValueFromFile(key, value))
   {
      audio_device(value.c_str());
   }
   else
   {
      popup_device_dialog(toplevel);
   }

   XtAppMainLoop(app);
}

void
expose_callback(
   Widget  w,
   caddr_t client_data,
   XmDrawingAreaCallbackStruct  *draw_struct)
{
}

void
gui_postit(
   Widget pb,
   void *popup,
   XEvent  *event,
   Boolean              *flag)
{
   if (((XButtonPressedEvent *)event)->button != Button1)
      return;

   XmMenuPosition((Widget)popup, (XButtonPressedEvent *)event);
   XtManageChild((Widget)popup);
}

void
scale_callback(
   Widget  w,
   caddr_t client_data,
   XmScaleCallbackStruct *scale_struct)
{
   if (scale_struct)
   {
      audio_set_position(scale_struct->value);
   }
}


void
gui_set_slider_position(
   long long cur_pos,
   long long total_pos,
   long long sam_per_sec)
{
   Arg  args[20];
   int  n;
   int value;
   static int previous_value = 0;

   if (total_pos == 0)
   {
      value = 0;
   }
   else
   {
      value = (int)((100 * cur_pos) / total_pos);
   }

   if (value != previous_value)
   {
      n = 0;
      XtSetArg(args[n], XmNvalue, value); n++;
      XtSetValues(PositionScale, args, n);
      previous_value = value;
   }

   if (sam_per_sec != 0)
   {
      _lbdraw->setTime((int)(cur_pos / sam_per_sec));
   }
}

void
gui_set_record_mode(int record)
{
   _lbdraw->setRecordMode(record);
}


